/*
 * (c) Copyright 1993, Silicon Graphics, Inc.
 * ALL RIGHTS RESERVED 
 * Permission to use, copy, modify, and distribute this software for 
 * any purpose and without fee is hereby granted, provided that the above
 * copyright notice appear in all copies and that both the copyright notice
 * and this permission notice appear in supporting documentation, and that 
 * the name of Silicon Graphics, Inc. not be used in advertising
 * or publicity pertaining to distribution of the software without specific,
 * written prior permission. 
 *
 * THE MATERIAL EMBODIED ON THIS SOFTWARE IS PROVIDED TO YOU "AS-IS"
 * AND WITHOUT WARRANTY OF ANY KIND, EXPRESS, IMPLIED OR OTHERWISE,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY OR
 * FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL SILICON
 * GRAPHICS, INC.  BE LIABLE TO YOU OR ANYONE ELSE FOR ANY DIRECT,
 * SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY
 * KIND, OR ANY DAMAGES WHATSOEVER, INCLUDING WITHOUT LIMITATION,
 * LOSS OF PROFIT, LOSS OF USE, SAVINGS OR REVENUE, OR THE CLAIMS OF
 * THIRD PARTIES, WHETHER OR NOT SILICON GRAPHICS, INC.  HAS BEEN
 * ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE
 * POSSESSION, USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 * US Government Users Restricted Rights 
 * Use, duplication, or disclosure by the Government is subject to
 * restrictions set forth in FAR 52.227.19(c)(2) or subparagraph
 * (c)(1)(ii) of the Rights in Technical Data and Computer Software
 * clause at DFARS 252.227-7013 and/or in similar or successor
 * clauses in the FAR or the DOD or NASA FAR Supplement.
 * Unpublished-- rights reserved under the copyright laws of the
 * United States.  Contractor/manufacturer is Silicon Graphics,
 * Inc., 2011 N.  Shoreline Blvd., Mountain View, CA 94039-7311.
 *
 * OpenGL(TM) is a trademark of Silicon Graphics, Inc.
 */
/*
 * material.c
 * This program demonstrates the use of the GL lighting model.
 * Several objects are drawn using different material characteristics.
 * A single light source illuminates the objects.
 */


// Modified by Sebastien Dedieu for Java
// October 1997.


import java.awt.*;
import java.awt.event.*;


public class Material extends java.applet.Applet
{
  MaterialCanvas materialCanvas;

  public void init()
  {
    setBackground(Color.black);

    materialCanvas=new MaterialCanvas(false);
    add(materialCanvas);
  }

  public static void main(String arg[])
  {
    System.loadLibrary("Meva");

    Frame materialFrame=new Frame("Material");
    materialFrame.resize(600, 450);
    materialFrame.setBackground(Color.black);

    MaterialCanvas materialCanvas=new MaterialCanvas(true);
    // materialFrame.addWindowListener(materialCanvas);

    materialFrame.add(materialCanvas);
    materialFrame.show();
  }

  public void paint(Graphics g)
  {
    materialCanvas.paint(g);
  }
}


class MaterialCanvas extends Canvas // implements WindowListener
{
  /*  Initialize z-buffer, projection matrix, light source, 
   *  and lighting model.  Do not specify a material property here.
   */

  boolean standAloneMode=false;
  boolean initialized=false;

  void myinit()
  {
      float ambient[] = { 0.0f, 0.0f, 0.0f, 1.0f };
      float diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
      float specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
      float position[] = { 0.0f, 3.0f, 2.0f, 0.0f };
      float lmodel_ambient[] = { 0.4f, 0.4f, 0.4f, 1.0f };
      float local_view[] = { 0.0f };

      Gl.glEnable(Gl.GL_DEPTH_TEST);
      Gl.glDepthFunc(Gl.GL_LESS);
  
      Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_AMBIENT, ambient);
      Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_DIFFUSE, diffuse);
      Gl.glLightfv(Gl.GL_LIGHT0, Gl.GL_POSITION, position);
      Gl.glLightModelfv(Gl.GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
      Gl.glLightModelfv(Gl.GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);
  
      Gl.glEnable(Gl.GL_LIGHTING);
      Gl.glEnable(Gl.GL_LIGHT0);
  
      Gl.glClearColor(0.0f, 0.1f, 0.1f, 0.0f);

      initialized=true;
  }
  
  /*  Draw twelve spheres in 3 rows with 4 columns.  
   *  The spheres in the first row have materials with no ambient reflection.
   *  The second row has materials with significant ambient reflection.
   *  The third row has materials with colored ambient reflection.
   *
   *  The first column has materials with blue, diffuse reflection only.
   *  The second column has blue diffuse reflection, as well as specular
   *  reflection with a low shininess exponent.
   *  The third column has blue diffuse reflection, as well as specular
   *  reflection with a high shininess exponent (a more concentrated highlight).
   *  The fourth column has materials which also include an emissive component.
   *
   *  glTranslatef() is used to move spheres to their appropriate locations.
   */
  
  void display()
  {
      float no_mat[] = { 0.0f, 0.0f, 0.0f, 1.0f };
      float mat_ambient[] = { 0.7f, 0.7f, 0.7f, 1.0f };
      float mat_ambient_color[] = { 0.8f, 0.8f, 0.2f, 1.0f };
      float mat_diffuse[] = { 0.1f, 0.5f, 0.8f, 1.0f };
      float mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
      float no_shininess[] = { 0.0f };
      float low_shininess[] = { 5.0f };
      float high_shininess[] = { 100.0f };
      float mat_emission[] = {0.3f, 0.2f, 0.2f, 0.0f};
  
      Gl.glClear(Gl.GL_COLOR_BUFFER_BIT | Gl.GL_DEPTH_BUFFER_BIT);
  
  /*  draw sphere in first row, first column
   *  diffuse reflection only; no ambient or specular  
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (-3.75f, 3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, no_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in first row, second column
   *  diffuse and specular reflection; low shininess; no ambient
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (-1.25f, 3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat_specular);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, low_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in first row, third column
   *  diffuse and specular reflection; high shininess; no ambient
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (1.25f, 3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat_specular);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, high_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in first row, fourth column
   *  diffuse reflection; emission; no ambient or specular reflection
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (3.75f, 3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, no_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, mat_emission);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in second row, first column
   *  ambient and diffuse reflection; no specular  
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (-3.75f, 0.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, no_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in second row, second column
   *  ambient, diffuse and specular reflection; low shininess
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (-1.25f, 0.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat_specular);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, low_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in second row, third column
   *  ambient, diffuse and specular reflection; high shininess
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (1.25f, 0.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat_specular);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, high_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in second row, fourth column
   *  ambient and diffuse reflection; emission; no specular
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (3.75f, 0.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, no_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, mat_emission);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in third row, first column
   *  colored ambient and diffuse reflection; no specular  
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (-3.75f, -3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient_color);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, no_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in third row, second column
   *  colored ambient, diffuse and specular reflection; low shininess
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (-1.25f, -3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient_color);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat_specular);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, low_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in third row, third column
   *  colored ambient, diffuse and specular reflection; high shininess
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (1.25f, -3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient_color);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, mat_specular);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, high_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, no_mat);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
  /*  draw sphere in third row, fourth column
   *  colored ambient and diffuse reflection; emission; no specular
   */
      Gl.glPushMatrix();
      Gl.glTranslatef (3.75f, -3.0f, 0.0f);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_AMBIENT, mat_ambient_color);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_DIFFUSE, mat_diffuse);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SPECULAR, no_mat);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_SHININESS, no_shininess);
      Gl.glMaterialfv(Gl.GL_FRONT, Gl.GL_EMISSION, mat_emission);
      Aux.auxSolidSphere(1.0);
      Gl.glPopMatrix();
  
      Gl.glFlush();
      Meva.swapBuffer();
  }
  
  void myReshape(int w, int h)
  {
      Gl.glViewport(0, 0, w, h);
      Gl.glMatrixMode(Gl.GL_PROJECTION);
      Gl.glLoadIdentity();
      if (w <= (h * 2))
  	Gl.glOrtho (-6.0, 6.0, -3.0*((float)h*2)/(float)w, 
  	    3.0*((float)h*2)/(float)w, -10.0, 10.0);
      else
  	Gl.glOrtho (-6.0*(float)w/((float)h*2), 
  	    6.0*(float)w/((float)h*2), -3.0, 3.0, -10.0, 10.0);
      Gl.glMatrixMode(Gl.GL_MODELVIEW);
  }
    
  public MaterialCanvas(boolean standAlone)
  {
    standAloneMode=standAlone;
  }

  Dimension oldDimension=new Dimension();

  public void paint(Graphics g)
  {
    if (!initialized)
    {
      if (!Meva.createContext(standAloneMode ? "Material" : "Netscape", 600, 450, 0.9f))
      {
        System.err.println("No context, I can't continue.");
        System.exit(0);
      }

      myinit();

      if (!standAloneMode)
	myReshape(600, 450);
    }

    Dimension dimension=size();

    if (!dimension.equals(oldDimension) && standAloneMode)
    {
      oldDimension=dimension;
      myReshape(dimension.width, dimension.height);
    }

    display();
  }

  /*

  public void windowOpened(WindowEvent e)
  {
  }

  public void windowClosing(WindowEvent e)
  {
    System.exit(0);
  }

  public void windowClosed(WindowEvent e)
  {
  }

  public void windowIconified(WindowEvent e)
  {
  }

  public void windowDeiconified(WindowEvent e)
  {
  }

  public void windowActivated(WindowEvent e)
  {
  }
 
  public void windowDeactivated(WindowEvent e)
  {
  }

  */
}








