/*
 * Meva
 * Version:  0.1
 * Copyright (C) 1997  Dedieu Sebastien
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdlib.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <X11/Xlib.h>
#include <X11/Xatom.h>
#include <X11/Xmu/WinUtil.h>
#include <GL/glx.h>
#include "Meva.h"
#include "MevaNativeDefinitions.h"


static Display *display=NULL;
static Window window=0;
static int found=0;


static void getAreaWindowAux(Window *window, int width, int height, float ratio, int *depth)
{
  Window *children, tmp1, tmp2;
  unsigned int numChildren;
  int frameDepth=*depth;

  if (XQueryTree (display, *window, &tmp1, &tmp2, &children, &numChildren)!=0)
  {
    int childCount;

    for (childCount=numChildren-1; childCount>=0; childCount--)
    {
      Window childWindow=children[childCount];
      XWindowAttributes windowAttributes;
      
      XGetWindowAttributes (display, childWindow, &windowAttributes);
      
      if (windowAttributes.width>=width*ratio && windowAttributes.height>=height*ratio)
      {
	int childDepth=frameDepth+1;

	getAreaWindowAux(&childWindow, width, height, ratio, &childDepth);
      
	if (*depth<childDepth)
        {
	  *depth=childDepth;
	  *window=childWindow;
	}
      }
    }
  }
}


static Window getAreaWindow(const char *title, int width, int height, float ratio)
{
  Window *children, tmp1, tmp2;
  unsigned int numChildren;
  Status result;

  result=XQueryTree(display,
		    RootWindowOfScreen(DefaultScreenOfDisplay(display)),
		    &tmp1, &tmp2,
		    &children,
		    &numChildren);

  assert(result!=0);
  
  if (numChildren!=0)
  {
    int childCount;

    for (childCount=numChildren-1; childCount>=0; childCount--)
    {
      Window childWindow=XmuClientWindow(display, children[childCount]);
      unsigned char *childTitle;
      Atom tmp1;
      int tmp2;
      unsigned long tmp3, tmp4;
      
      XGetWindowProperty(display,
			 childWindow,
			 XA_WM_NAME,
			 0,
			 256,
			 0,
			 XA_STRING,
			 &tmp1,
			 &tmp2,
			 &tmp3,
			 &tmp4,
			 &childTitle);

      if (strstr(childTitle, title)!=0)
      {
	int depth=0;

	getAreaWindowAux(&childWindow, width, height, ratio, &depth);

	return childWindow;
      }
    }
  }

  return 0;
}


static int createContext(const char *title, int width, int height, float ratio)
{
   GLXContext glxContext;
   XVisualInfo *visualInfo;
   int attributes[] = { GLX_RGBA,
			GLX_RED_SIZE, 1,
			GLX_GREEN_SIZE, 1,
			GLX_BLUE_SIZE, 1,
			GLX_DOUBLEBUFFER,
			GLX_DEPTH_SIZE, 1,
			0 };

   assert(!found);

   if (display==0)
     display=XOpenDisplay(NULL);
   assert(display!=0);

   window=getAreaWindow(title, width, height, ratio);
   if (window==0)
     return 0;

   visualInfo=glXChooseVisual(display, DefaultScreen(display), attributes);

   if (visualInfo==0)
   {
     printf("Couldn't get an RGB, Double-buffered visual.\n");

     return 0;
   }

  glxContext=glXCreateContext(display, visualInfo, NULL, True);

  if (visualInfo==0)
  {
    printf("Couldn't create a glX context.\n");
    
    return 0;
  }

  glXMakeCurrent(display, window, glxContext);

  found=1;

  fprintf(stderr, "\n");
  fprintf(stderr, "Meva 0.1\n");
  fprintf(stderr, "  by Sebastien Dedieu (sebastien.dedieu@hol.fr)\n");
  fprintf(stderr, "  October 1997.\n");
  fprintf(stderr, "\n");

  return 1;
}


static void swapBuffer()
{
  assert(found);

  glXSwapBuffers(display, window);
}


MEVA_NATIVE(jboolean, Meva_createContext)(MEVA_ENV *_env, jclass _class, jstring title, jint width, jint height, float ratio)
{
  jboolean _returned_value;
  const char *_title=MEVA_GET_STRING_UTF(_env, title, 0);

  _returned_value=(jboolean)createContext((const char *)_title, (int)width, (int)height, (float)ratio);

  MEVA_RELEASE_STRING_UTF(_env, title, _title);

  return _returned_value;
}


MEVA_NATIVE(void, Meva_swapBuffer)(MEVA_ENV *_env, jclass _class)
{
  swapBuffer();
}












