/*
 * Meva
 * Version:  0.1
 * Copyright (C) 1997  Dedieu Sebastien
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free
 * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include <stdio.h>
#include "npapi.h"


#define IMPLEMENT_Gl
#define IMPLEMENT_Glu
#define IMPLEMENT_Aux
#define IMPLEMENT_Meva


#include "Gl.h"
#include "Glu.h"
#include "Aux.h"
#include "Meva.h"


typedef struct _PluginInstance
{
  int nothing;
} PluginInstance;


char*NPP_GetMIMEDescription(void)
{
  return "application/x-meva-plugin:meva:Meva plugin";
}


NPError NPP_GetValue(void *future, NPPVariable variable, void *value)
{
  NPError err=NPERR_NO_ERROR;
  
  switch (variable)
  {
    case NPPVpluginNameString:
    *((char **)value)="Meva plugin";
    break;
  
    case NPPVpluginDescriptionString:
    *((char **)value)="This plugins allows Open GL calls within a Java applet."; 
    break;
  
    default:
    err=NPERR_GENERIC_ERROR;
  }

  return err;
}


NPError NPP_Initialize(void)
{
  return NPERR_NO_ERROR;
}


JRIEnv *env=NULL;


jref NPP_GetJavaClass()
{
  env=NPN_GetJavaEnv();

  if (env==NULL)
    return NULL;

  use_Gl(env);
  use_Glu(env);
  use_Aux(env);
  use_Meva(env);

  return NULL;
}


void NPP_Shutdown(void)
{
  if (env!=NULL)
  {
    unuse_Gl(env);
    unuse_Glu(env);
    unuse_Aux(env);
    unuse_Meva(env);
  }
}


NPError NPP_New(NPMIMEType pluginType,
		NPP instance,
		uint16 mode,
		int16 argc,
		char* argn[],
		char* argv[],
		NPSavedData* saved)
{
  PluginInstance* This;

  if (instance==NULL)
    return NPERR_INVALID_INSTANCE_ERROR;
  
  instance->pdata=NPN_MemAlloc(sizeof(PluginInstance));
  
  This=(PluginInstance*)instance->pdata;
  
  if (This!=NULL)
    return NPERR_NO_ERROR;
  else
    return NPERR_OUT_OF_MEMORY_ERROR;
}


NPError NPP_Destroy(NPP instance, NPSavedData** save)
{
  PluginInstance* This;

  if (instance==NULL)
    return NPERR_INVALID_INSTANCE_ERROR;

  This=(PluginInstance*)instance->pdata;
  
  if (This!=NULL)
  {
    NPN_MemFree(instance->pdata);
    instance->pdata=NULL;
  }

  return NPERR_NO_ERROR;
}



NPError NPP_SetWindow(NPP instance, NPWindow* window)
{
  PluginInstance* This;

  if (instance==NULL)
    return NPERR_INVALID_INSTANCE_ERROR;

  if (window==NULL)
    return NPERR_NO_ERROR;

  This=(PluginInstance*)instance->pdata;

  return NPERR_NO_ERROR;
}


NPError NPP_NewStream(NPP instance,
		      NPMIMEType type,
		      NPStream *stream, 
		      NPBool seekable,
		      uint16 *stype)
{
  NPByteRange range;
  PluginInstance* This;

  if (instance==NULL)
    return NPERR_INVALID_INSTANCE_ERROR;

  This=(PluginInstance*)instance->pdata;
  
  return NPERR_NO_ERROR;
}


int32 STREAMBUFSIZE=0X0FFFFFFF;


int32 NPP_WriteReady(NPP instance, NPStream *stream)
{
  PluginInstance* This;
  if (instance!=NULL)
    This=(PluginInstance*)instance->pdata;
  
  return STREAMBUFSIZE;
}


int32 NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
  if (instance!=NULL)
  {
    PluginInstance* This=(PluginInstance*)instance->pdata;
  }

  return len;
}


NPError 
NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
{
  PluginInstance* This;
  
  if (instance==NULL)
    return NPERR_INVALID_INSTANCE_ERROR;

  This=(PluginInstance*)instance->pdata;
  
  return NPERR_NO_ERROR;
}


void NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
  PluginInstance* This;
  if (instance!=NULL)
    This=(PluginInstance*)instance->pdata;
}


void NPP_Print(NPP instance, NPPrint* printInfo)
{
  if(printInfo==NULL)
    return;
  
  if (instance!=NULL)
  {
    PluginInstance* This=(PluginInstance*)instance->pdata;
	
    if (printInfo->mode==NP_FULL)
    {
      void* platformPrint=printInfo->print.fullPrint.platformPrint;
      NPBool printOne=printInfo->print.fullPrint.printOne;
      
      printInfo->print.fullPrint.pluginPrinted=FALSE;
    }
    else
    {
      NPWindow* printWindow=&(printInfo->print.embedPrint.window);
      void* platformPrint=printInfo->print.embedPrint.platformPrint;
    }
  }
}
